package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"path/filepath"
)

func findNext(filename string) (string, error) {
	fin, err := os.Open(filename)
	if err != nil {
		return "", err
	}
	r := bufio.NewReader(fin)

	minimum, err := r.ReadString('\n')
	if err == io.EOF {
		return "", err
	} else if err != nil {
		return "", err
	}

	temp := filename + ".temp"
	fout, err := os.OpenFile(temp, os.O_RDWR|os.O_CREATE, 0660)
	if err != nil {
		return "", err
	}

	for {
		line, err := r.ReadString('\n')
		if err == io.EOF {
			break
		} else if err != nil {
			return "", err
		}

		if line < minimum {
			temp := minimum
			minimum = line
			line = temp
		}
		fmt.Fprintf(fout, "%s", line)
	}

	fin.Close()
	fout.Close()
	err = os.Rename(temp, filename)
	if err != nil {
		fmt.Println(err)
		return "", err
	}

	return minimum, nil
}

func main() {

	if len(os.Args) != 2 {
		fmt.Printf("usage: %s inputfile\n", filepath.Base(os.Args[0]))
		os.Exit(1)
	}

	filename := os.Args[1]
	sorted := filename + ".sorted"
	f, err := os.OpenFile(sorted, os.O_RDWR|os.O_APPEND|os.O_CREATE, 0660)
	if err != nil {
		fmt.Println(err)
		os.Exit(-1)
	}
	defer f.Close()

	for {
		nextLine, err := findNext(filename)
		if err != nil {
			fmt.Println(err)
			break
		}
		fmt.Fprintf(f, "%s", nextLine)
	}

	err = os.Rename(sorted, filename)
	if err != nil {
		fmt.Println(err)
		os.Exit(-1)
	}
}
