package main

import (
	"bufio"
	"fmt"
	"io"
	"os"
	"path/filepath"
	"strconv"
	"strings"
)

var MAX int
var MIN int

func findNext(filename string, size int, space int) error {
	data := make([]int, size)
	fin, err := os.Open(filename)
	if err != nil {
		return err
	}
	defer fin.Close()
	r := bufio.NewReader(fin)
	for {
		line, err := r.ReadString('\n')
		if err == io.EOF {
			break
		} else if err != nil {
			return err
		}

		line = strings.Trim(line, "\n")
		t8, _ := strconv.ParseUint(line, 10, 32)
		valueToExamine := int(t8)
		if valueToExamine >= space && valueToExamine < space+size {
			index := valueToExamine - space
			data[index] = 1
		}
	}

	for x := range data {
		if data[x] == 1 {
			value := space + x
			fmt.Println(value)
		}
	}
	return nil
}

func main() {
	arguments := os.Args
	if len(arguments) != 3 {
		fmt.Printf("usage: %s SIZE inputfile\n", filepath.Base(os.Args[0]))
		os.Exit(1)
	}

	t, _ := strconv.ParseUint(arguments[1], 10, 32)
	size := int(t)
	filename := arguments[2]

	fin, err := os.Open(filename)
	if err != nil {
		fmt.Println(err)
		os.Exit(1)
	}
	r := bufio.NewReader(fin)
	line, err := r.ReadString('\n')
	line = strings.Trim(line, "\n")
	if err == io.EOF {
		fmt.Println(err)
		os.Exit(1)
	} else if err != nil {
		fmt.Println(err)
		os.Exit(1)
	}

	temp, _ := strconv.ParseUint(line, 10, 32)
	MIN = int(temp)
	MAX = int(temp)

	for {
		line, err := r.ReadString('\n')
		line = strings.Trim(line, "\n")
		if err == io.EOF {
			break
		} else if err != nil {
			fmt.Println(err)
			os.Exit(1)
		}
		t, _ := strconv.ParseUint(line, 10, 32)
		temp := int(t)

		if temp < MIN {
			MIN = temp
		}

		if temp > MAX {
			MAX = temp
		}
	}
	fin.Close()
	fmt.Println(MIN, MAX)
	var numberOfPasses int = (MAX-MIN)/size + 1
	fmt.Println("Passes:", numberOfPasses)

	i := 0
	for i < numberOfPasses {
		err := findNext(filename, size, MIN+i*size)
		if err != nil {
			fmt.Println(err)
			break
		}
		i = i + 1
	}

}
